@@ -30,10 +30,9 @@ GST_DEBUG_CATEGORY_STATIC (gst_cuda_base_convert_debug);
 #define GST_CAT_DEFAULT gst_cuda_base_convert_debug
 
 #define GST_CUDA_CONVET_FORMATS \
-    "{ I420, YV12, NV12, NV21, P010_10LE, P012_LE, P016_LE, I420_10LE, I420_12LE, Y444, " \
-    "Y444_10LE, Y444_12LE, Y444_16LE, BGRA, RGBA, RGBx, BGRx, ARGB, ABGR, RGB, " \
-    "BGR, BGR10A2_LE, RGB10A2_LE, Y42B, I422_10LE, I422_12LE, RGBP, BGRP, GBR, " \
-    "GBRA, GBR_10LE, GBR_12LE, GBR_16LE, VUYA }"
+    "{ I420, YV12, NV12, NV21, P010_10LE, P016_LE, I420_10LE, Y444, Y444_16LE, " \
+    "BGRA, RGBA, RGBx, BGRx, ARGB, ABGR, RGB, BGR, BGR10A2_LE, RGB10A2_LE, " \
+    "Y42B, I422_10LE, I422_12LE, RGBP, BGRP, GBR, GBRA }"
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -56,30 +55,13 @@ struct _GstCudaBaseConvert
   GstCudaBaseTransform parent;
 
   GstCudaConverter *converter;
-  GstCudaStream *other_stream;
 
   gint borders_h;
   gint borders_w;
   gboolean add_borders;
-  gboolean downstream_supports_crop_meta;
-  gboolean same_caps;
-  GstVideoRectangle in_rect;
-
-  /* orientation */
-  /* method configured via property */
-  GstVideoOrientationMethod method;
-  /* method parsed from tag */
-  GstVideoOrientationMethod tag_method;
-  /* method currently selected based on "method" and "tag_method" */
-  GstVideoOrientationMethod selected_method;
-  /* method previously selected and used for negotiation */
-  GstVideoOrientationMethod active_method;
-
-  GMutex lock;
 };
 
 static void gst_cuda_base_convert_dispose (GObject * object);
-static void gst_cuda_base_convert_finalize (GObject * object);
 static GstCaps *gst_cuda_base_convert_transform_caps (GstBaseTransform * trans,
     GstPadDirection direction, GstCaps * caps, GstCaps * filter);
 static GstCaps *gst_cuda_base_convert_fixate_caps (GstBaseTransform * trans,
@@ -89,12 +71,10 @@ gst_cuda_base_convert_propose_allocation (GstBaseTransform * trans,
     GstQuery * decide_query, GstQuery * query);
 static gboolean gst_cuda_base_convert_decide_allocation (GstBaseTransform *
     trans, GstQuery * query);
-static gboolean gst_cuda_base_convert_transform_meta (GstBaseTransform * trans,
-    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf);
+static gboolean gst_cuda_base_convert_filter_meta (GstBaseTransform * trans,
+    GstQuery * query, GType api, const GstStructure * params);
 static GstFlowReturn gst_cuda_base_convert_transform (GstBaseTransform * trans,
     GstBuffer * inbuf, GstBuffer * outbuf);
-static GstFlowReturn gst_cuda_base_convert_generate_output (GstBaseTransform *
-    trans, GstBuffer ** buffer);
 static gboolean gst_cuda_base_convert_set_info (GstCudaBaseTransform * btrans,
     GstCaps * incaps, GstVideoInfo * in_info, GstCaps * outcaps,
     GstVideoInfo * out_info);
@@ -122,12 +102,11 @@ gst_cuda_base_convert_class_init (GstCudaBaseConvertClass * klass)
       GST_CUDA_BASE_TRANSFORM_CLASS (klass);
 
   gobject_class->dispose = gst_cuda_base_convert_dispose;
-  gobject_class->finalize = gst_cuda_base_convert_finalize;
 
   gst_element_class_add_static_pad_template (element_class, &sink_template);
   gst_element_class_add_static_pad_template (element_class, &src_template);
 
-  trans_class->passthrough_on_same_caps = FALSE;
+  trans_class->passthrough_on_same_caps = TRUE;
 
   trans_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_cuda_base_convert_transform_caps);
@@ -137,11 +116,9 @@ gst_cuda_base_convert_class_init (GstCudaBaseConvertClass * klass)
       GST_DEBUG_FUNCPTR (gst_cuda_base_convert_propose_allocation);
   trans_class->decide_allocation =
       GST_DEBUG_FUNCPTR (gst_cuda_base_convert_decide_allocation);
-  trans_class->transform_meta =
-      GST_DEBUG_FUNCPTR (gst_cuda_base_convert_transform_meta);
+  trans_class->filter_meta =
+      GST_DEBUG_FUNCPTR (gst_cuda_base_convert_filter_meta);
   trans_class->transform = GST_DEBUG_FUNCPTR (gst_cuda_base_convert_transform);
-  trans_class->generate_output =
-      GST_DEBUG_FUNCPTR (gst_cuda_base_convert_generate_output);
 
   btrans_class->set_info = GST_DEBUG_FUNCPTR (gst_cuda_base_convert_set_info);
 
@@ -152,7 +129,6 @@ static void
 gst_cuda_base_convert_init (GstCudaBaseConvert * self)
 {
   self->add_borders = DEFAULT_ADD_BORDERS;
-  g_mutex_init (&self->lock);
 }
 
 static void
@@ -160,22 +136,11 @@ gst_cuda_base_convert_dispose (GObject * object)
 {
   GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (object);
 
-  gst_clear_cuda_stream (&self->other_stream);
   gst_clear_object (&self->converter);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static void
-gst_cuda_base_convert_finalize (GObject * object)
-{
-  GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (object);
-
-  g_mutex_clear (&self->lock);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
 static GstCaps *
 gst_cuda_base_convert_caps_remove_format_info (GstCaps * caps)
 {
@@ -184,8 +149,7 @@ gst_cuda_base_convert_caps_remove_format_info (GstCaps * caps)
   gint i, n;
   GstCaps *res;
   GstCapsFeatures *feature =
-      gst_caps_features_new_single_static_str
-      (GST_CAPS_FEATURE_MEMORY_CUDA_MEMORY);
+      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_CUDA_MEMORY);
 
   res = gst_caps_new_empty ();
 
@@ -222,8 +186,7 @@ gst_cuda_base_convert_caps_rangify_size_info (GstCaps * caps)
   gint i, n;
   GstCaps *res;
   GstCapsFeatures *feature =
-      gst_caps_features_new_single_static_str
-      (GST_CAPS_FEATURE_MEMORY_CUDA_MEMORY);
+      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_CUDA_MEMORY);
 
   res = gst_caps_new_empty ();
 
@@ -266,8 +229,7 @@ gst_cuda_base_convert_caps_remove_format_and_rangify_size_info (GstCaps * caps)
   gint i, n;
   GstCaps *res;
   GstCapsFeatures *feature =
-      gst_caps_features_new_single_static_str
-      (GST_CAPS_FEATURE_MEMORY_CUDA_MEMORY);
+      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_CUDA_MEMORY);
 
   res = gst_caps_new_empty ();
 
@@ -646,11 +608,9 @@ static GstCaps *
 gst_cuda_base_convert_fixate_size (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps, GstCaps * othercaps)
 {
-  GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (base);
   GstStructure *ins, *outs;
   const GValue *from_par, *to_par;
   GValue fpar = G_VALUE_INIT, tpar = G_VALUE_INIT;
-  gboolean rotate = FALSE;
 
   othercaps = gst_caps_truncate (othercaps);
   othercaps = gst_caps_make_writable (othercaps);
@@ -660,19 +620,6 @@ gst_cuda_base_convert_fixate_size (GstBaseTransform * base,
   from_par = gst_structure_get_value (ins, "pixel-aspect-ratio");
   to_par = gst_structure_get_value (outs, "pixel-aspect-ratio");
 
-  g_mutex_lock (&self->lock);
-  switch (self->selected_method) {
-    case GST_VIDEO_ORIENTATION_90R:
-    case GST_VIDEO_ORIENTATION_90L:
-    case GST_VIDEO_ORIENTATION_UL_LR:
-    case GST_VIDEO_ORIENTATION_UR_LL:
-      rotate = TRUE;
-      break;
-    default:
-      rotate = FALSE;
-      break;
-  }
-
   if (direction == GST_PAD_SINK) {
     if (!from_par) {
       g_value_init (&fpar, GST_TYPE_FRACTION);
@@ -701,13 +648,8 @@ gst_cuda_base_convert_fixate_size (GstBaseTransform * base,
     if (!to_par) {
       gint to_par_n, to_par_d;
 
-      if (rotate) {
-        to_par_n = from_par_n;
-        to_par_d = from_par_d;
-      } else {
-        to_par_n = from_par_n;
-        to_par_d = from_par_d;
-      }
+      to_par_n = from_par_n;
+      to_par_d = from_par_d;
 
       g_value_init (&tpar, GST_TYPE_FRACTION);
       gst_value_set_fraction (&tpar, to_par_n, to_par_d);
@@ -737,17 +679,6 @@ gst_cuda_base_convert_fixate_size (GstBaseTransform * base,
     gst_structure_get_int (outs, "width", &w);
     gst_structure_get_int (outs, "height", &h);
 
-    /* swap dimensions when it's rotated */
-    if (rotate) {
-      gint _tmp = from_w;
-      from_w = from_h;
-      from_h = _tmp;
-
-      _tmp = from_par_n;
-      from_par_n = from_par_d;
-      from_par_d = _tmp;
-    }
-
     /* if both width and height are already fixed, we can't do anything
      * about it anymore */
     if (w && h) {
@@ -1113,7 +1044,6 @@ done:
     g_value_unset (&fpar);
   if (to_par == &tpar)
     g_value_unset (&tpar);
-  g_mutex_unlock (&self->lock);
 
   return othercaps;
 }
@@ -1169,7 +1099,6 @@ static gboolean
 gst_cuda_base_convert_propose_allocation (GstBaseTransform * trans,
     GstQuery * decide_query, GstQuery * query)
 {
-  GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (trans);
   GstCudaBaseTransform *ctrans = GST_CUDA_BASE_TRANSFORM (trans);
   GstVideoInfo info;
   GstBufferPool *pool;
@@ -1180,11 +1109,9 @@ gst_cuda_base_convert_propose_allocation (GstBaseTransform * trans,
           decide_query, query))
     return FALSE;
 
-  if (self->same_caps && gst_pad_peer_query (trans->srcpad, query)) {
-    gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
-    gst_query_add_allocation_meta (query, GST_VIDEO_CROP_META_API_TYPE, NULL);
+  /* passthrough, we're done */
+  if (decide_query == NULL)
     return TRUE;
-  }
 
   gst_query_parse_allocation (query, &caps, NULL);
 
@@ -1200,14 +1127,6 @@ gst_cuda_base_convert_propose_allocation (GstBaseTransform * trans,
     pool = gst_cuda_buffer_pool_new (ctrans->context);
 
     config = gst_buffer_pool_get_config (pool);
-    /* Forward downstream CUDA stream to upstream */
-    if (self->other_stream) {
-      GST_DEBUG_OBJECT (self, "Have downstream CUDA stream, forwarding");
-      gst_buffer_pool_config_set_cuda_stream (config, self->other_stream);
-    } else if (ctrans->stream) {
-      GST_DEBUG_OBJECT (self, "Set our stream to proposing buffer pool");
-      gst_buffer_pool_config_set_cuda_stream (config, ctrans->stream);
-    }
 
     gst_buffer_pool_config_add_option (config,
         GST_BUFFER_POOL_OPTION_VIDEO_META);
@@ -1232,7 +1151,6 @@ gst_cuda_base_convert_propose_allocation (GstBaseTransform * trans,
   }
 
   gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
-  gst_query_add_allocation_meta (query, GST_VIDEO_CROP_META_API_TYPE, NULL);
 
   return TRUE;
 }
@@ -1241,7 +1159,6 @@ static gboolean
 gst_cuda_base_convert_decide_allocation (GstBaseTransform * trans,
     GstQuery * query)
 {
-  GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (trans);
   GstCudaBaseTransform *ctrans = GST_CUDA_BASE_TRANSFORM (trans);
   GstCaps *outcaps = NULL;
   GstBufferPool *pool = NULL;
@@ -1254,11 +1171,6 @@ gst_cuda_base_convert_decide_allocation (GstBaseTransform * trans,
   if (!outcaps)
     return FALSE;
 
-  self->downstream_supports_crop_meta = gst_query_find_allocation_meta (query,
-      GST_VIDEO_CROP_META_API_TYPE, NULL);
-  GST_DEBUG_OBJECT (self, "Downstream crop meta support: %d",
-      self->downstream_supports_crop_meta);
-
   if (gst_query_get_n_allocation_pools (query) > 0) {
     gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
     if (pool) {
@@ -1290,15 +1202,6 @@ gst_cuda_base_convert_decide_allocation (GstBaseTransform * trans,
   config = gst_buffer_pool_get_config (pool);
   gst_buffer_pool_config_add_option (config, GST_BUFFER_POOL_OPTION_VIDEO_META);
   gst_buffer_pool_config_set_params (config, outcaps, size, min, max);
-  gst_clear_cuda_stream (&self->other_stream);
-  self->other_stream = gst_buffer_pool_config_get_cuda_stream (config);
-  if (self->other_stream) {
-    GST_DEBUG_OBJECT (self, "Downstream provided CUDA stream");
-  } else if (ctrans->stream) {
-    GST_DEBUG_OBJECT (self, "Set our stream to decided buffer pool");
-    gst_buffer_pool_config_set_cuda_stream (config, ctrans->stream);
-  }
-
   gst_buffer_pool_set_config (pool, config);
 
   /* Get updated size by cuda buffer pool */
@@ -1317,32 +1220,6 @@ gst_cuda_base_convert_decide_allocation (GstBaseTransform * trans,
       query);
 }
 
-static gboolean
-needs_color_convert (const GstVideoInfo * in_info,
-    const GstVideoInfo * out_info)
-{
-  const GstVideoColorimetry *in_cinfo = &in_info->colorimetry;
-  const GstVideoColorimetry *out_cinfo = &out_info->colorimetry;
-
-  if (in_cinfo->range != out_cinfo->range ||
-      in_cinfo->matrix != out_cinfo->matrix) {
-    return TRUE;
-  }
-
-  if (!gst_video_color_primaries_is_equivalent (in_cinfo->primaries,
-          out_cinfo->primaries)) {
-    return TRUE;
-  }
-
-  if (!gst_video_transfer_function_is_equivalent (in_cinfo->transfer,
-          GST_VIDEO_INFO_COMP_DEPTH (in_info, 0), out_cinfo->transfer,
-          GST_VIDEO_INFO_COMP_DEPTH (out_info, 0))) {
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 static gboolean
 gst_cuda_base_convert_set_info (GstCudaBaseTransform * btrans,
     GstCaps * incaps, GstVideoInfo * in_info, GstCaps * outcaps,
@@ -1350,45 +1227,13 @@ gst_cuda_base_convert_set_info (GstCudaBaseTransform * btrans,
 {
   GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (btrans);
   gint from_dar_n, from_dar_d, to_dar_n, to_dar_d;
-  gboolean need_flip = FALSE;
-  gint in_width, in_height, in_par_n, in_par_d;
-  GstVideoOrientationMethod active_method;
+  GstVideoInfo tmp_info;
 
   gst_clear_object (&self->converter);
 
-  g_mutex_lock (&self->lock);
-  active_method = self->active_method = self->selected_method;
-  g_mutex_unlock (&self->lock);
-
-  if (active_method != GST_VIDEO_ORIENTATION_IDENTITY)
-    need_flip = TRUE;
-
-  if (!need_flip && gst_caps_is_equal (incaps, outcaps)) {
-    self->same_caps = TRUE;
-  } else {
-    self->same_caps = FALSE;
-  }
-
-  switch (active_method) {
-    case GST_VIDEO_ORIENTATION_90R:
-    case GST_VIDEO_ORIENTATION_90L:
-    case GST_VIDEO_ORIENTATION_UL_LR:
-    case GST_VIDEO_ORIENTATION_UR_LL:
-      in_width = in_info->height;
-      in_height = in_info->width;
-      in_par_n = in_info->par_d;
-      in_par_d = in_info->par_n;
-      break;
-    default:
-      in_width = in_info->width;
-      in_height = in_info->height;
-      in_par_n = in_info->par_n;
-      in_par_d = in_info->par_d;
-      break;
-  }
-
-  if (!gst_util_fraction_multiply (in_width,
-          in_height, in_par_n, in_par_d, &from_dar_n, &from_dar_d)) {
+  if (!gst_util_fraction_multiply (in_info->width,
+          in_info->height, in_info->par_n, in_info->par_d, &from_dar_n,
+          &from_dar_d)) {
     from_dar_n = from_dar_d = -1;
   }
 
@@ -1420,7 +1265,7 @@ gst_cuda_base_convert_set_info (GstCudaBaseTransform * btrans,
         GST_WARNING_OBJECT (self, "Can't calculate borders");
       }
     } else {
-      GST_DEBUG_OBJECT (self, "Can't keep DAR!");
+      GST_WARNING_OBJECT (self, "Can't keep DAR!");
     }
   }
 
@@ -1430,25 +1275,36 @@ gst_cuda_base_convert_set_info (GstCudaBaseTransform * btrans,
     return FALSE;
   }
 
-  if (in_width == out_info->width && in_height == out_info->height
-      && in_info->finfo == out_info->finfo && self->borders_w == 0 &&
-      self->borders_h == 0 && !need_flip &&
-      !needs_color_convert (in_info, out_info)) {
-    self->same_caps = TRUE;
-  }
+  /* if the only thing different in the caps is the transfer function, and
+   * we're converting between equivalent transfer functions, do passthrough */
+  tmp_info = *in_info;
+  tmp_info.colorimetry.transfer = out_info->colorimetry.transfer;
+  if (gst_video_info_is_equal (&tmp_info, out_info) &&
+      gst_video_transfer_function_is_equivalent (in_info->colorimetry.transfer,
+          in_info->finfo->bits, out_info->colorimetry.transfer,
+          out_info->finfo->bits)) {
+    gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (self), TRUE);
+  } else {
+    GstStructure *config;
 
-  self->converter = gst_cuda_converter_new (in_info,
-      out_info, btrans->context, NULL);
-  if (!self->converter) {
-    GST_ERROR_OBJECT (self, "Couldn't create converter");
-    return FALSE;
-  }
+    gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (self), FALSE);
 
-  g_object_set (self->converter, "dest-x", self->borders_w / 2,
-      "dest-y", self->borders_h / 2,
-      "dest-width", out_info->width - self->borders_w,
-      "dest-height", out_info->height - self->borders_h,
-      "fill-border", TRUE, "video-direction", active_method, NULL);
+    config = gst_structure_new_empty ("GstCudaConverter");
+    gst_structure_set (config,
+        GST_CUDA_CONVERTER_OPT_DEST_X, G_TYPE_INT, self->borders_w / 2,
+        GST_CUDA_CONVERTER_OPT_DEST_Y, G_TYPE_INT, self->borders_h / 2,
+        GST_CUDA_CONVERTER_OPT_DEST_WIDTH,
+        G_TYPE_INT, out_info->width - self->borders_w,
+        GST_CUDA_CONVERTER_OPT_DEST_HEIGHT,
+        G_TYPE_INT, out_info->height - self->borders_h, NULL);
+
+    self->converter = gst_cuda_converter_new (in_info,
+        out_info, btrans->context, config);
+    if (!self->converter) {
+      GST_ERROR_OBJECT (self, "Couldn't create converter");
+      return FALSE;
+    }
+  }
 
   GST_DEBUG_OBJECT (self, "%s from=%dx%d (par=%d/%d dar=%d/%d), size %"
       G_GSIZE_FORMAT " -> %s to=%dx%d (par=%d/%d dar=%d/%d borders=%d:%d), "
@@ -1461,23 +1317,21 @@ gst_cuda_base_convert_set_info (GstCudaBaseTransform * btrans,
       out_info->height, out_info->par_n, out_info->par_d, to_dar_n, to_dar_d,
       self->borders_w, self->borders_h, out_info->size);
 
-  self->in_rect.x = 0;
-  self->in_rect.y = 0;
-  self->in_rect.w = in_info->width;
-  self->in_rect.h = in_info->height;
-
   return TRUE;
 }
 
 static gboolean
-gst_cuda_base_convert_transform_meta (GstBaseTransform * trans,
-    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf)
+gst_cuda_base_convert_filter_meta (GstBaseTransform * trans, GstQuery * query,
+    GType api, const GstStructure * params)
 {
-  if (meta->info->api == GST_VIDEO_CROP_META_API_TYPE)
+  /* This element cannot passthrough the crop meta, because it would convert the
+   * wrong sub-region of the image, and worst, our output image may not be large
+   * enough for the crop to be applied later */
+  if (api == GST_VIDEO_CROP_META_API_TYPE)
     return FALSE;
 
-  return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (trans,
-      outbuf, meta, inbuf);
+  /* propose all other metadata upstream */
+  return TRUE;
 }
 
 static GstFlowReturn
@@ -1489,24 +1343,6 @@ gst_cuda_base_convert_transform (GstBaseTransform * trans,
   GstVideoFrame in_frame, out_frame;
   GstFlowReturn ret = GST_FLOW_OK;
   GstMemory *mem;
-  GstCudaMemory *in_cmem, *out_cmem;
-  GstCudaStream *in_stream, *out_stream;
-  GstCudaStream *selected_stream = NULL;
-  gboolean sync_done = FALSE;
-  GstVideoRectangle in_rect;
-
-  GstVideoCropMeta *crop_meta = gst_buffer_get_video_crop_meta (inbuf);
-  if (crop_meta) {
-    in_rect.x = crop_meta->x;
-    in_rect.y = crop_meta->y;
-    in_rect.w = crop_meta->width;
-    in_rect.h = crop_meta->height;
-  } else {
-    in_rect = self->in_rect;
-  }
-
-  g_object_set (self->converter, "src-x", in_rect.x, "src-y", in_rect.y,
-      "src-width", in_rect.w, "src-height", in_rect.h, NULL);
 
   if (gst_buffer_n_memory (inbuf) != 1) {
     GST_ERROR_OBJECT (self, "Invalid input buffer");
@@ -1518,8 +1354,6 @@ gst_cuda_base_convert_transform (GstBaseTransform * trans,
     GST_ERROR_OBJECT (self, "Input buffer is not CUDA");
     return GST_FLOW_ERROR;
   }
-  in_cmem = GST_CUDA_MEMORY_CAST (mem);
-  in_stream = gst_cuda_memory_get_stream (in_cmem);
 
   if (gst_buffer_n_memory (outbuf) != 1) {
     GST_ERROR_OBJECT (self, "Invalid output buffer");
@@ -1531,8 +1365,6 @@ gst_cuda_base_convert_transform (GstBaseTransform * trans,
     GST_ERROR_OBJECT (self, "Input buffer is not CUDA");
     return GST_FLOW_ERROR;
   }
-  out_cmem = GST_CUDA_MEMORY_CAST (mem);
-  out_stream = gst_cuda_memory_get_stream (out_cmem);
 
   if (!gst_video_frame_map (&in_frame, &btrans->in_info, inbuf,
           GST_MAP_READ | GST_MAP_CUDA)) {
@@ -1547,79 +1379,18 @@ gst_cuda_base_convert_transform (GstBaseTransform * trans,
     return GST_FLOW_ERROR;
   }
 
-  /* If downstream does not aware of CUDA stream (i.e., using default stream) */
-  if (!out_stream) {
-    if (in_stream) {
-      GST_TRACE_OBJECT (self, "Use upstram CUDA stream");
-      selected_stream = in_stream;
-    } else if (btrans->stream) {
-      GST_TRACE_OBJECT (self, "Use our CUDA stream");
-      selected_stream = btrans->stream;
-    }
-  } else {
-    selected_stream = out_stream;
-    if (in_stream) {
-      if (in_stream == out_stream) {
-        GST_TRACE_OBJECT (self, "Same stream");
-      } else {
-        GST_TRACE_OBJECT (self, "Different CUDA stream");
-        gst_cuda_memory_sync (in_cmem);
-      }
-    }
-  }
-
   if (!gst_cuda_converter_convert_frame (self->converter, &in_frame, &out_frame,
-          gst_cuda_stream_get_handle (selected_stream), &sync_done)) {
+          btrans->cuda_stream)) {
     GST_ERROR_OBJECT (self, "Failed to convert frame");
     ret = GST_FLOW_ERROR;
   }
 
-  if (sync_done) {
-    GST_TRACE_OBJECT (self, "Sync done by converter");
-    GST_MEMORY_FLAG_UNSET (out_cmem, GST_CUDA_MEMORY_TRANSFER_NEED_SYNC);
-  } else if (selected_stream != out_stream) {
-    GST_MEMORY_FLAG_UNSET (out_cmem, GST_CUDA_MEMORY_TRANSFER_NEED_SYNC);
-    GST_TRACE_OBJECT (self, "Waiting for convert sync");
-    gst_cuda_context_push (btrans->context);
-    CuStreamSynchronize (gst_cuda_stream_get_handle (selected_stream));
-    gst_cuda_context_pop (NULL);
-  }
-
   gst_video_frame_unmap (&out_frame);
   gst_video_frame_unmap (&in_frame);
 
   return ret;
 }
 
-static GstFlowReturn
-gst_cuda_base_convert_generate_output (GstBaseTransform * trans,
-    GstBuffer ** buffer)
-{
-  GstCudaBaseConvert *self = GST_CUDA_BASE_CONVERT (trans);
-  gboolean passthrough = self->same_caps;
-
-  if (!trans->queued_buf)
-    return GST_FLOW_OK;
-
-  if (passthrough && !self->downstream_supports_crop_meta) {
-    if (gst_buffer_get_video_crop_meta (trans->queued_buf)) {
-      GST_LOG_OBJECT (self,
-          "Buffer has crop meta but downstream does not support crop");
-      passthrough = FALSE;
-    }
-  }
-
-  if (!passthrough) {
-    return GST_BASE_TRANSFORM_CLASS (parent_class)->generate_output (trans,
-        buffer);
-  }
-
-  *buffer = trans->queued_buf;
-  trans->queued_buf = NULL;
-
-  return GST_FLOW_OK;
-}
-
 static void
 gst_cuda_base_convert_set_add_border (GstCudaBaseConvert * self,
     gboolean add_border)
@@ -1631,37 +1402,6 @@ gst_cuda_base_convert_set_add_border (GstCudaBaseConvert * self,
     gst_base_transform_reconfigure_src (GST_BASE_TRANSFORM_CAST (self));
 }
 
-static void
-gst_cuda_base_convert_set_orientation (GstCudaBaseConvert * self,
-    GstVideoOrientationMethod method, gboolean from_tag)
-{
-  if (method == GST_VIDEO_ORIENTATION_CUSTOM) {
-    GST_WARNING_OBJECT (self, "Unsupported custom orientation");
-    return;
-  }
-
-  g_mutex_lock (&self->lock);
-  if (from_tag)
-    self->tag_method = method;
-  else
-    self->method = method;
-
-  if (self->method == GST_VIDEO_ORIENTATION_AUTO) {
-    self->selected_method = self->tag_method;
-  } else {
-    self->selected_method = self->method;
-  }
-
-  if (self->selected_method != self->active_method) {
-    GST_DEBUG_OBJECT (self, "Rotation orientation %d -> %d",
-        self->active_method, self->selected_method);
-
-    gst_base_transform_reconfigure_src (GST_BASE_TRANSFORM (self));
-  }
-
-  g_mutex_unlock (&self->lock);
-}
-
 /**
  * SECTION:element-cudaconvertscale
  * @title: cudaconvertscale
@@ -1685,7 +1425,6 @@ enum
 {
   PROP_CONVERT_SCALE_0,
   PROP_CONVERT_SCALE_ADD_BORDERS,
-  PROP_CONVERT_SCALE_VIDEO_DIRECTION,
 };
 
 struct _GstCudaConvertScale
@@ -1693,34 +1432,19 @@ struct _GstCudaConvertScale
   GstCudaBaseConvert parent;
 };
 
-static void
-    gst_cuda_convert_scale_video_direction_interface_init
-    (GstVideoDirectionInterface * iface)
-{
-}
-
 static void gst_cuda_convert_scale_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_cuda_convert_scale_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
-static gboolean gst_cuda_convert_scale_sink_event (GstBaseTransform * trans,
-    GstEvent * event);
 
-#define gst_cuda_convert_scale_parent_class convert_scale_parent_class
-G_DEFINE_TYPE_WITH_CODE (GstCudaConvertScale, gst_cuda_convert_scale,
-    GST_TYPE_CUDA_BASE_CONVERT,
-    G_IMPLEMENT_INTERFACE (GST_TYPE_VIDEO_DIRECTION,
-        gst_cuda_convert_scale_video_direction_interface_init));
-
-static void gst_cuda_convert_scale_before_transform (GstBaseTransform * trans,
-    GstBuffer * buffer);
+G_DEFINE_TYPE (GstCudaConvertScale, gst_cuda_convert_scale,
+    GST_TYPE_CUDA_BASE_CONVERT);
 
 static void
 gst_cuda_convert_scale_class_init (GstCudaConvertScaleClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  GstBaseTransformClass *transform_class = GST_BASE_TRANSFORM_CLASS (klass);
 
   gobject_class->set_property = gst_cuda_convert_scale_set_property;
   gobject_class->get_property = gst_cuda_convert_scale_get_property;
@@ -1732,27 +1456,11 @@ gst_cuda_convert_scale_class_init (GstCudaConvertScaleClass * klass)
           DEFAULT_ADD_BORDERS, (GParamFlags) (GST_PARAM_MUTABLE_PLAYING |
               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
-  /**
-   * GstCudaConvertScale:video-direction:
-   *
-   * Video rotation/flip method to use
-   *
-   * Since: 1.24
-   */
-  g_object_class_override_property (gobject_class,
-      PROP_CONVERT_SCALE_VIDEO_DIRECTION, "video-direction");
-
   gst_element_class_set_static_metadata (element_class,
       "CUDA colorspace converter and scaler",
-      "Filter/Converter/Video/Scaler/Colorspace/Effect/Hardware",
+      "Filter/Converter/Video/Scaler/Colorspace/Hardware",
       "Resizes video and allow color conversion using CUDA",
       "Seungha Yang <seungha@centricular.com>");
-
-  transform_class->passthrough_on_same_caps = FALSE;
-  transform_class->before_transform =
-      GST_DEBUG_FUNCPTR (gst_cuda_convert_scale_before_transform);
-  transform_class->sink_event =
-      GST_DEBUG_FUNCPTR (gst_cuda_convert_scale_sink_event);
 }
 
 static void
@@ -1770,10 +1478,6 @@ gst_cuda_convert_scale_set_property (GObject * object, guint prop_id,
     case PROP_CONVERT_SCALE_ADD_BORDERS:
       gst_cuda_base_convert_set_add_border (base, g_value_get_boolean (value));
       break;
-    case PROP_CONVERT_SCALE_VIDEO_DIRECTION:
-      gst_cuda_base_convert_set_orientation (base, g_value_get_enum (value),
-          FALSE);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1790,8 +1494,6 @@ gst_cuda_convert_scale_get_property (GObject * object, guint prop_id,
     case PROP_CONVERT_SCALE_ADD_BORDERS:
       g_value_set_boolean (value, base->add_borders);
       break;
-    case PROP_CONVERT_SCALE_VIDEO_DIRECTION:
-      g_value_set_enum (value, base->method);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1799,76 +1501,6 @@ gst_cuda_convert_scale_get_property (GObject * object, guint prop_id,
   }
 }
 
-static void
-gst_cuda_convert_scale_before_transform (GstBaseTransform * trans,
-    GstBuffer * buffer)
-{
-  GstCudaBaseConvert *base = GST_CUDA_BASE_CONVERT (trans);
-  gboolean update = FALSE;
-  GstCaps *in_caps;
-  GstCaps *out_caps;
-  GstBaseTransformClass *klass;
-
-  GST_BASE_TRANSFORM_CLASS (convert_scale_parent_class)->before_transform
-      (trans, buffer);
-
-  g_mutex_lock (&base->lock);
-  if (base->selected_method != base->active_method)
-    update = TRUE;
-  g_mutex_unlock (&base->lock);
-
-  if (!update)
-    return;
-
-  /* basetransform wouldn't call set_caps if in/out caps were not changed.
-   * Update it manually here */
-  GST_DEBUG_OBJECT (base, "Updating caps for direction change");
-
-  in_caps = gst_pad_get_current_caps (GST_BASE_TRANSFORM_SINK_PAD (trans));
-  if (!in_caps) {
-    GST_WARNING_OBJECT (trans, "sinkpad has no current caps");
-    return;
-  }
-
-  out_caps = gst_pad_get_current_caps (GST_BASE_TRANSFORM_SRC_PAD (trans));
-  if (!out_caps) {
-    GST_WARNING_OBJECT (trans, "srcpad has no current caps");
-    gst_caps_unref (in_caps);
-    return;
-  }
-
-  klass = GST_BASE_TRANSFORM_GET_CLASS (trans);
-  klass->set_caps (trans, in_caps, out_caps);
-  gst_caps_unref (in_caps);
-  gst_caps_unref (out_caps);
-
-  gst_base_transform_reconfigure_src (trans);
-}
-
-static gboolean
-gst_cuda_convert_scale_sink_event (GstBaseTransform * trans, GstEvent * event)
-{
-  GstCudaBaseConvert *base = GST_CUDA_BASE_CONVERT (trans);
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_TAG:{
-      GstTagList *taglist;
-      GstVideoOrientationMethod method = GST_VIDEO_ORIENTATION_IDENTITY;
-
-      gst_event_parse_tag (event, &taglist);
-      if (gst_video_orientation_from_tag (taglist, &method))
-        gst_cuda_base_convert_set_orientation (base, method, TRUE);
-      break;
-    }
-    default:
-      break;
-  }
-
-  return
-      GST_BASE_TRANSFORM_CLASS (convert_scale_parent_class)->sink_event
-      (trans, event);
-}
-
 /**
  * SECTION:element-cudaconvert
  * @title: cudaconvert
